@@ -6,7 +6,9 @@ package cache
 
 import (
 	"context"
+	"encoding/json"
 	"fmt"
+	"io"
 	"os"
 	"path/filepath"
 	"regexp"
@@ -29,45 +31,29 @@ const (
 
 type parseModHandle struct {
 	handle *memoize.Handle
-
-	mod, sum source.FileHandle
 }
 
 type parseModData struct {
-	memoize.NoCopy
-
-	parsed *modfile.File
-	m      *protocol.ColumnMapper
-
-	// parseErrors refers to syntax errors found in the go.mod file.
-	parseErrors []source.Error
+	parsed *source.ParsedModule
 
 	// err is any error encountered while parsing the file.
 	err error
 }
 
-func (mh *parseModHandle) Mod() source.FileHandle {
-	return mh.mod
-}
-
-func (mh *parseModHandle) Sum() source.FileHandle {
-	return mh.sum
-}
-
-func (mh *parseModHandle) Parse(ctx context.Context) (*modfile.File, *protocol.ColumnMapper, []source.Error, error) {
-	v, err := mh.handle.Get(ctx)
+func (mh *parseModHandle) parse(ctx context.Context, snapshot *snapshot) (*source.ParsedModule, error) {
+	v, err := mh.handle.Get(ctx, snapshot.generation, snapshot)
 	if err != nil {
-		return nil, nil, nil, err
+		return nil, err
 	}
 	data := v.(*parseModData)
-	return data.parsed, data.m, data.parseErrors, data.err
+	return data.parsed, data.err
 }
 
-func (s *snapshot) ParseModHandle(ctx context.Context, modFH source.FileHandle) (source.ParseModHandle, error) {
-	if handle := s.getModHandle(modFH.URI()); handle != nil {
-		return handle, nil
+func (s *snapshot) ParseMod(ctx context.Context, modFH source.FileHandle) (*source.ParsedModule, error) {
+	if handle := s.getParseModHandle(modFH.URI()); handle != nil {
+		return handle.parse(ctx, s)
 	}
-	h := s.view.session.cache.store.Bind(modFH.Identity().String(), func(ctx context.Context) interface{} {
+	h := s.generation.Bind(modFH.FileIdentity(), func(ctx context.Context, _ memoize.Arg) interface{} {
 		_, done := event.Start(ctx, "cache.ParseModHandle", tag.URI.Of(modFH.URI()))
 		defer done()
 
@@ -80,55 +66,57 @@ func (s *snapshot) ParseModHandle(ctx context.Context, modFH source.FileHandle)
 			Converter: span.NewContentConverter(modFH.URI().Filename(), contents),
 			Content:   contents,
 		}
-		parsed, err := modfile.Parse(modFH.URI().Filename(), contents, nil)
-		if err != nil {
-			parseErr, _ := extractModParseErrors(modFH.URI(), m, err, contents)
-			var parseErrors []source.Error
-			if parseErr != nil {
-				parseErrors = append(parseErrors, *parseErr)
+		data := &parseModData{
+			parsed: &source.ParsedModule{
+				Mapper: m,
+			},
+		}
+		data.parsed.File, data.err = modfile.Parse(modFH.URI().Filename(), contents, nil)
+		if data.err != nil {
+			// Attempt to convert the error to a standardized parse error.
+			if parseErr, extractErr := extractModParseErrors(modFH.URI(), m, data.err, contents); extractErr == nil {
+				data.parsed.ParseErrors = []source.Error{*parseErr}
 			}
-			return &parseModData{
-				parseErrors: parseErrors,
-				err:         err,
+			// If the file was still parsed, we don't want to treat this as a
+			// fatal error. Note: This currently cannot happen as modfile.Parse
+			// always returns an error when the file is nil.
+			if data.parsed.File != nil {
+				data.err = nil
 			}
 		}
-		return &parseModData{
-			parsed: parsed,
-			m:      m,
-		}
+		return data
 	})
+
+	pmh := &parseModHandle{handle: h}
+	s.mu.Lock()
+	s.parseModHandles[modFH.URI()] = pmh
+	s.mu.Unlock()
+
+	return pmh.parse(ctx, s)
+}
+
+func (s *snapshot) sumFH(ctx context.Context, modFH source.FileHandle) (source.FileHandle, error) {
 	// Get the go.sum file, either from the snapshot or directly from the
 	// cache. Avoid (*snapshot).GetFile here, as we don't want to add
 	// nonexistent file handles to the snapshot if the file does not exist.
 	sumURI := span.URIFromPath(sumFilename(modFH.URI()))
-	sumFH := s.FindFile(sumURI)
+	var sumFH source.FileHandle = s.FindFile(sumURI)
 	if sumFH == nil {
-		fh, err := s.view.session.cache.getFile(ctx, sumURI)
-		if err != nil && !os.IsNotExist(err) {
+		var err error
+		sumFH, err = s.view.session.cache.getFile(ctx, sumURI)
+		if err != nil {
 			return nil, err
 		}
-		if fh.err != nil && !os.IsNotExist(fh.err) {
-			return nil, fh.err
-		}
-		// If the file doesn't exist, we can just keep the go.sum nil.
-		if err != nil || fh.err != nil {
-			sumFH = nil
-		} else {
-			sumFH = fh
-		}
 	}
-	s.mu.Lock()
-	defer s.mu.Unlock()
-	s.parseModHandles[modFH.URI()] = &parseModHandle{
-		handle: h,
-		mod:    modFH,
-		sum:    sumFH,
+	_, err := sumFH.Read()
+	if err != nil {
+		return nil, err
 	}
-	return s.parseModHandles[modFH.URI()], nil
+	return sumFH, nil
 }
 
 func sumFilename(modURI span.URI) string {
-	return modURI.Filename()[:len(modURI.Filename())-len("mod")] + "sum"
+	return strings.TrimSuffix(modURI.Filename(), ".mod") + ".sum"
 }
 
 // extractModParseErrors processes the raw errors returned by modfile.Parse,
@@ -174,8 +162,9 @@ func extractModParseErrors(uri span.URI, m *protocol.ColumnMapper, parseErr erro
 // modKey is uniquely identifies cached data for `go mod why` or dependencies
 // to upgrade.
 type modKey struct {
-	sessionID, cfg, mod, view string
-	verb                      modAction
+	sessionID, cfg, view string
+	mod                  source.FileIdentity
+	verb                 modAction
 }
 
 type modAction int
@@ -187,8 +176,6 @@ const (
 
 type modWhyHandle struct {
 	handle *memoize.Handle
-
-	pmh source.ParseModHandle
 }
 
 type modWhyData struct {
@@ -199,8 +186,8 @@ type modWhyData struct {
 	err error
 }
 
-func (mwh *modWhyHandle) Why(ctx context.Context) (map[string]string, error) {
-	v, err := mwh.handle.Get(ctx)
+func (mwh *modWhyHandle) why(ctx context.Context, snapshot *snapshot) (map[string]string, error) {
+	v, err := mwh.handle.Get(ctx, snapshot.generation, snapshot)
 	if err != nil {
 		return nil, err
 	}
@@ -208,75 +195,71 @@ func (mwh *modWhyHandle) Why(ctx context.Context) (map[string]string, error) {
 	return data.why, data.err
 }
 
-func (s *snapshot) ModWhyHandle(ctx context.Context) (source.ModWhyHandle, error) {
+func (s *snapshot) ModWhy(ctx context.Context, fh source.FileHandle) (map[string]string, error) {
+	if fh.Kind() != source.Mod {
+		return nil, fmt.Errorf("%s is not a go.mod file", fh.URI())
+	}
 	if err := s.awaitLoaded(ctx); err != nil {
 		return nil, err
 	}
-	fh, err := s.GetFile(ctx, s.view.modURI)
-	if err != nil {
-		return nil, err
+	if handle := s.getModWhyHandle(fh.URI()); handle != nil {
+		return handle.why(ctx, s)
 	}
-	pmh, err := s.ParseModHandle(ctx, fh)
-	if err != nil {
-		return nil, err
-	}
-	var (
-		cfg    = s.config(ctx)
-		tmpMod = s.view.tmpMod
-	)
+	// Make sure to use the module root as the working directory.
+	cfg := s.configWithDir(ctx, filepath.Dir(fh.URI().Filename()))
 	key := modKey{
 		sessionID: s.view.session.id,
 		cfg:       hashConfig(cfg),
-		mod:       pmh.Mod().Identity().String(),
-		view:      s.view.root.Filename(),
+		mod:       fh.FileIdentity(),
+		view:      s.view.rootURI.Filename(),
 		verb:      why,
 	}
-	h := s.view.session.cache.store.Bind(key, func(ctx context.Context) interface{} {
-		ctx, done := event.Start(ctx, "cache.ModHandle", tag.URI.Of(pmh.Mod().URI()))
+	h := s.generation.Bind(key, func(ctx context.Context, arg memoize.Arg) interface{} {
+		ctx, done := event.Start(ctx, "cache.ModWhyHandle", tag.URI.Of(fh.URI()))
 		defer done()
 
-		parsed, _, _, err := pmh.Parse(ctx)
+		snapshot := arg.(*snapshot)
+
+		pm, err := snapshot.ParseMod(ctx, fh)
 		if err != nil {
 			return &modWhyData{err: err}
 		}
 		// No requires to explain.
-		if len(parsed.Require) == 0 {
+		if len(pm.File.Require) == 0 {
 			return &modWhyData{}
 		}
 		// Run `go mod why` on all the dependencies.
 		args := []string{"why", "-m"}
-		for _, req := range parsed.Require {
+		for _, req := range pm.File.Require {
 			args = append(args, req.Mod.Path)
 		}
-		_, stdout, err := runGoCommand(ctx, cfg, pmh, tmpMod, "mod", args)
+		stdout, err := snapshot.runGoCommandWithConfig(ctx, cfg, "mod", args)
 		if err != nil {
 			return &modWhyData{err: err}
 		}
 		whyList := strings.Split(stdout.String(), "\n\n")
-		if len(whyList) != len(parsed.Require) {
+		if len(whyList) != len(pm.File.Require) {
 			return &modWhyData{
-				err: fmt.Errorf("mismatched number of results: got %v, want %v", len(whyList), len(parsed.Require)),
+				err: fmt.Errorf("mismatched number of results: got %v, want %v", len(whyList), len(pm.File.Require)),
 			}
 		}
-		why := make(map[string]string, len(parsed.Require))
-		for i, req := range parsed.Require {
+		why := make(map[string]string, len(pm.File.Require))
+		for i, req := range pm.File.Require {
 			why[req.Mod.Path] = whyList[i]
 		}
 		return &modWhyData{why: why}
 	})
+
+	mwh := &modWhyHandle{handle: h}
 	s.mu.Lock()
-	defer s.mu.Unlock()
-	s.modWhyHandle = &modWhyHandle{
-		handle: h,
-		pmh:    pmh,
-	}
-	return s.modWhyHandle, nil
+	s.modWhyHandles[fh.URI()] = mwh
+	s.mu.Unlock()
+
+	return mwh.why(ctx, s)
 }
 
 type modUpgradeHandle struct {
 	handle *memoize.Handle
-
-	pmh source.ParseModHandle
 }
 
 type modUpgradeData struct {
@@ -286,8 +269,8 @@ type modUpgradeData struct {
 	err error
 }
 
-func (muh *modUpgradeHandle) Upgrades(ctx context.Context) (map[string]string, error) {
-	v, err := muh.handle.Get(ctx)
+func (muh *modUpgradeHandle) upgrades(ctx context.Context, snapshot *snapshot) (map[string]string, error) {
+	v, err := muh.handle.Get(ctx, snapshot.generation, snapshot)
 	if v == nil {
 		return nil, err
 	}
@@ -295,84 +278,92 @@ func (muh *modUpgradeHandle) Upgrades(ctx context.Context) (map[string]string, e
 	return data.upgrades, data.err
 }
 
-func (s *snapshot) ModUpgradeHandle(ctx context.Context) (source.ModUpgradeHandle, error) {
+// moduleUpgrade describes a module that can be upgraded to a particular
+// version.
+type moduleUpgrade struct {
+	Path   string
+	Update struct {
+		Version string
+	}
+}
+
+func (s *snapshot) ModUpgrade(ctx context.Context, fh source.FileHandle) (map[string]string, error) {
+	if fh.Kind() != source.Mod {
+		return nil, fmt.Errorf("%s is not a go.mod file", fh.URI())
+	}
 	if err := s.awaitLoaded(ctx); err != nil {
 		return nil, err
 	}
-	fh, err := s.GetFile(ctx, s.view.modURI)
-	if err != nil {
-		return nil, err
+	if handle := s.getModUpgradeHandle(fh.URI()); handle != nil {
+		return handle.upgrades(ctx, s)
 	}
-	pmh, err := s.ParseModHandle(ctx, fh)
-	if err != nil {
-		return nil, err
-	}
-	var (
-		cfg    = s.config(ctx)
-		tmpMod = s.view.tmpMod
-	)
+	// Use the module root as the working directory.
+	cfg := s.configWithDir(ctx, filepath.Dir(fh.URI().Filename()))
 	key := modKey{
 		sessionID: s.view.session.id,
 		cfg:       hashConfig(cfg),
-		mod:       pmh.Mod().Identity().String(),
-		view:      s.view.root.Filename(),
+		mod:       fh.FileIdentity(),
+		view:      s.view.rootURI.Filename(),
 		verb:      upgrade,
 	}
-	h := s.view.session.cache.store.Bind(key, func(ctx context.Context) interface{} {
-		ctx, done := event.Start(ctx, "cache.ModUpgradeHandle", tag.URI.Of(pmh.Mod().URI()))
+	h := s.generation.Bind(key, func(ctx context.Context, arg memoize.Arg) interface{} {
+		ctx, done := event.Start(ctx, "cache.ModUpgradeHandle", tag.URI.Of(fh.URI()))
 		defer done()
 
-		parsed, _, _, err := pmh.Parse(ctx)
+		snapshot := arg.(*snapshot)
+
+		pm, err := snapshot.ParseMod(ctx, fh)
 		if err != nil {
 			return &modUpgradeData{err: err}
 		}
+
 		// No requires to upgrade.
-		if len(parsed.Require) == 0 {
+		if len(pm.File.Require) == 0 {
 			return &modUpgradeData{}
 		}
 		// Run "go list -mod readonly -u -m all" to be able to see which deps can be
 		// upgraded without modifying mod file.
-		args := []string{"-u", "-m", "all"}
-		if !tmpMod || containsVendor(pmh.Mod().URI()) {
+		args := []string{"-u", "-m", "-json", "all"}
+		if s.view.workspaceMode&tempModfile == 0 || containsVendor(fh.URI()) {
 			// Use -mod=readonly if the module contains a vendor directory
 			// (see golang/go#38711).
 			args = append([]string{"-mod", "readonly"}, args...)
 		}
-		_, stdout, err := runGoCommand(ctx, cfg, pmh, tmpMod, "list", args)
+		stdout, err := snapshot.runGoCommandWithConfig(ctx, cfg, "list", args)
 		if err != nil {
 			return &modUpgradeData{err: err}
 		}
-		upgradesList := strings.Split(stdout.String(), "\n")
-		if len(upgradesList) <= 1 {
-			return nil
+		var upgradeList []moduleUpgrade
+		dec := json.NewDecoder(stdout)
+		for {
+			var m moduleUpgrade
+			if err := dec.Decode(&m); err == io.EOF {
+				break
+			} else if err != nil {
+				return &modUpgradeData{err: err}
+			}
+			upgradeList = append(upgradeList, m)
+		}
+		if len(upgradeList) <= 1 {
+			return &modUpgradeData{}
 		}
 		upgrades := make(map[string]string)
-		for _, upgrade := range upgradesList[1:] {
-			// Example: "github.com/x/tools v1.1.0 [v1.2.0]"
-			info := strings.Split(upgrade, " ")
-			if len(info) != 3 {
+		for _, upgrade := range upgradeList[1:] {
+			if upgrade.Update.Version == "" {
 				continue
 			}
-			dep, version := info[0], info[2]
-
-			// Make sure that the format matches our expectation.
-			if version[0] != '[' || version[len(version)-1] != ']' {
-				continue
-			}
-			latest := version[1 : len(version)-1] // remove the "[" and "]"
-			upgrades[dep] = latest
+			upgrades[upgrade.Path] = upgrade.Update.Version
 		}
 		return &modUpgradeData{
 			upgrades: upgrades,
 		}
 	})
+	muh := &modUpgradeHandle{handle: h}
 	s.mu.Lock()
-	defer s.mu.Unlock()
-	s.modUpgradeHandle = &modUpgradeHandle{
-		handle: h,
-		pmh:    pmh,
-	}
-	return s.modUpgradeHandle, nil
+	s.modUpgradeHandles[fh.URI()] = muh
+	s.mu.Unlock()
+
+	return muh.upgrades(ctx, s)
 }
 
 // containsVendor reports whether the module has a vendor folder.
