@@ -16,16 +16,18 @@ import (
 	"path"
 	"path/filepath"
 	"reflect"
+	"regexp"
 	"sort"
 	"strings"
 	"sync"
 	"time"
 
+	"golang.org/x/mod/modfile"
+	"golang.org/x/mod/semver"
 	"golang.org/x/tools/internal/event"
 	"golang.org/x/tools/internal/event/keys"
 	"golang.org/x/tools/internal/gocommand"
 	"golang.org/x/tools/internal/imports"
-	"golang.org/x/tools/internal/lsp/debug/tag"
 	"golang.org/x/tools/internal/lsp/source"
 	"golang.org/x/tools/internal/memoize"
 	"golang.org/x/tools/internal/span"
@@ -38,7 +40,7 @@ type View struct {
 	id      string
 
 	optionsMu sync.Mutex
-	options   source.Options
+	options   *source.Options
 
 	// mu protects most mutable state of the view.
 	mu sync.Mutex
@@ -61,10 +63,6 @@ type View struct {
 	// folder is the folder with which this view was constructed.
 	folder span.URI
 
-	// root is the root directory of this view. If we are in GOPATH mode, this
-	// is just the folder. If we are in module mode, this is the module root.
-	root span.URI
-
 	// importsMu guards imports-related state, particularly the ProcessEnv.
 	importsMu sync.Mutex
 
@@ -116,16 +114,22 @@ type View struct {
 	initializeOnce     *sync.Once
 	initializedErr     error
 
+	// workspaceInformation tracks various details about this view's
+	// environment variables, go version, and use of modules.
+	workspaceInformation
+
+	// workspaceMode describes the way in which the view's workspace should be
+	// loaded.
+	workspaceMode workspaceMode
+
 	// True if the view is either in GOPATH, a module, or some other
 	// non go command build system.
 	hasValidBuildConfiguration bool
+}
 
-	// The real go.mod and go.sum files that are attributed to a view.
-	modURI, sumURI span.URI
-
-	// True if this view runs go commands using temporary mod files.
-	// Only possible with Go versions 1.14 and above.
-	tmpMod bool
+type workspaceInformation struct {
+	// The Go version in use: X in Go 1.X.
+	goversion int
 
 	// hasGopackagesDriver is true if the user has a value set for the
 	// GOPACKAGESDRIVER environment variable or a gopackagesdriver binary on
@@ -133,13 +137,40 @@ type View struct {
 	hasGopackagesDriver bool
 
 	// `go env` variables that need to be tracked by gopls.
-	gocache, gomodcache, gopath, goprivate string
+	environmentVariables
+
+	// The value of GO111MODULE we want to run with.
+	go111module string
 
 	// goEnv is the `go env` output collected when a view is created.
 	// It includes the values of the environment variables above.
 	goEnv map[string]string
+
+	// The real go.mod and go.sum files that are attributed to a view.
+	modURI, sumURI span.URI
+
+	// rootURI is the rootURI directory of this view. If we are in GOPATH mode, this
+	// is just the folder. If we are in module mode, this is the module rootURI.
+	rootURI span.URI
 }
 
+type environmentVariables struct {
+	gocache, gopath, goprivate, gomodcache, gomod string
+}
+
+type workspaceMode int
+
+const (
+	moduleMode workspaceMode = 1 << iota
+
+	// tempModfile indicates whether or not the -modfile flag should be used.
+	tempModfile
+
+	// usesWorkspaceModule indicates support for the experimental workspace module
+	// feature.
+	usesWorkspaceModule
+)
+
 type builtinPackageHandle struct {
 	handle *memoize.Handle
 }
@@ -149,6 +180,11 @@ type builtinPackageData struct {
 	err    error
 }
 
+type moduleRoot struct {
+	rootURI        span.URI
+	modURI, sumURI span.URI
+}
+
 // fileBase holds the common functionality for all files.
 // It is intended to be embedded in the file implementations
 type fileBase struct {
@@ -245,13 +281,13 @@ func (v *View) Folder() span.URI {
 	return v.folder
 }
 
-func (v *View) Options() source.Options {
+func (v *View) Options() *source.Options {
 	v.optionsMu.Lock()
 	defer v.optionsMu.Unlock()
 	return v.options
 }
 
-func minorOptionsChange(a, b source.Options) bool {
+func minorOptionsChange(a, b *source.Options) bool {
 	// Check if any of the settings that modify our understanding of files have been changed
 	mapEnv := func(env []string) map[string]string {
 		m := make(map[string]string, len(env))
@@ -261,7 +297,6 @@ func minorOptionsChange(a, b source.Options) bool {
 				continue
 			}
 			m[split[0]] = split[1]
-
 		}
 		return m
 	}
@@ -283,7 +318,7 @@ func minorOptionsChange(a, b source.Options) bool {
 	return true
 }
 
-func (v *View) SetOptions(ctx context.Context, options source.Options) (source.View, error) {
+func (v *View) SetOptions(ctx context.Context, options *source.Options) (source.View, error) {
 	// no need to rebuild the view if the options were not materially changed
 	v.optionsMu.Lock()
 	if minorOptionsChange(v.options, options) {
@@ -324,8 +359,26 @@ func (v *View) WriteEnv(ctx context.Context, w io.Writer) error {
 		}
 
 	}
-	fmt.Fprintf(w, "go env for %v\n(root %s)\n(valid build configuration = %v)\n(build flags: %v)\n",
-		v.folder.Filename(), v.root.Filename(), v.hasValidBuildConfiguration, buildFlags)
+	goVersion, err := v.session.gocmdRunner.Run(ctx, gocommand.Invocation{
+		Verb:       "version",
+		BuildFlags: buildFlags,
+		Env:        env,
+		WorkingDir: v.rootURI.Filename(),
+	})
+	if err != nil {
+		return err
+	}
+	fmt.Fprintf(w, `go env for %v
+(root %s)
+(go version %s)
+(valid build configuration = %v)
+(build flags: %v)
+`,
+		v.folder.Filename(),
+		v.rootURI.Filename(),
+		goVersion.String(),
+		v.hasValidBuildConfiguration,
+		buildFlags)
 	for k, v := range fullEnv {
 		fmt.Fprintf(w, "%s=%s\n", k, v)
 	}
@@ -358,7 +411,7 @@ func (v *View) RunProcessEnvFunc(ctx context.Context, fn func(*imports.Options)
 	// v.goEnv is immutable -- changes make a new view. Options can change.
 	// We can't compare build flags directly because we may add -modfile.
 	v.optionsMu.Lock()
-	localPrefix := v.options.LocalPrefix
+	localPrefix := v.options.Local
 	currentBuildFlags := v.options.BuildFlags
 	changed := !reflect.DeepEqual(currentBuildFlags, v.cachedBuildFlags) ||
 		v.options.VerboseOutput != (v.processEnv.Logf != nil) ||
@@ -456,8 +509,24 @@ func (v *View) populateProcessEnv(ctx context.Context, modFH, sumFH source.FileH
 	}
 	v.optionsMu.Unlock()
 
+	pe.Env = map[string]string{}
+	for k, v := range v.goEnv {
+		pe.Env[k] = v
+	}
+	pe.Env["GO111MODULE"] = v.go111module
+
+	modmod, err := v.needsModEqualsMod(ctx, modFH)
+	if err != nil {
+		return cleanup, err
+	}
+	if modmod {
+		// -mod isn't really a build flag, but we can get away with it given
+		// the set of commands that goimports wants to run.
+		pe.BuildFlags = append([]string{"-mod=mod"}, pe.BuildFlags...)
+	}
+
 	// Add -modfile to the build flags, if we are using it.
-	if v.tmpMod && modFH != nil {
+	if v.workspaceMode&tempModfile != 0 && modFH != nil {
 		var tmpURI span.URI
 		tmpURI, cleanup, err = tempModFile(modFH, sumFH)
 		if err != nil {
@@ -472,13 +541,13 @@ func (v *View) populateProcessEnv(ctx context.Context, modFH, sumFH source.FileH
 // envLocked returns the environment and build flags for the current view.
 // It assumes that the caller is holding the view's optionsMu.
 func (v *View) envLocked() ([]string, []string) {
-	env := append([]string{}, v.options.Env...)
+	env := append(os.Environ(), v.options.Env...)
 	buildFlags := append([]string{}, v.options.BuildFlags...)
 	return env, buildFlags
 }
 
 func (v *View) contains(uri span.URI) bool {
-	return strings.HasPrefix(string(uri), string(v.root))
+	return strings.HasPrefix(string(uri), string(v.rootURI))
 }
 
 func (v *View) mapFile(uri span.URI, f *fileBase) {
@@ -631,62 +700,81 @@ func checkIgnored(suffix string) bool {
 }
 
 func (v *View) Snapshot(ctx context.Context) (source.Snapshot, func()) {
-	s := v.getSnapshot()
-	return s, s.generation.Acquire(ctx)
-}
-
-func (v *View) getSnapshot() *snapshot {
 	v.snapshotMu.Lock()
 	defer v.snapshotMu.Unlock()
-
-	return v.snapshot
+	return v.snapshot, v.snapshot.generation.Acquire(ctx)
 }
 
-func (v *View) initialize(ctx context.Context, s *snapshot, firstAttempt bool) {
+func (s *snapshot) initialize(ctx context.Context, firstAttempt bool) {
 	select {
 	case <-ctx.Done():
 		return
-	case v.initializationSema <- struct{}{}:
+	case s.view.initializationSema <- struct{}{}:
 	}
 
 	defer func() {
-		<-v.initializationSema
+		<-s.view.initializationSema
 	}()
 
-	if v.initializeOnce == nil {
+	if s.view.initializeOnce == nil {
 		return
 	}
-	v.initializeOnce.Do(func() {
+	s.view.initializeOnce.Do(func() {
 		defer func() {
-			v.initializeOnce = nil
+			s.view.initializeOnce = nil
 			if firstAttempt {
-				close(v.initialized)
+				close(s.view.initialized)
 			}
 		}()
 
-		err := s.load(ctx, viewLoadScope("LOAD_VIEW"), packagePath("builtin"))
+		// If we have multiple modules, we need to load them by paths.
+		var scopes []interface{}
+		var modErrors *source.ErrorList
+		addError := func(uri span.URI, err error) {
+			if modErrors == nil {
+				modErrors = &source.ErrorList{}
+			}
+			*modErrors = append(*modErrors, &source.Error{
+				URI:      uri,
+				Category: "compiler",
+				Kind:     source.ListError,
+				Message:  err.Error(),
+			})
+		}
+		if len(s.modules) > 0 {
+			for _, mod := range s.modules {
+				fh, err := s.GetFile(ctx, mod.modURI)
+				if err != nil {
+					addError(mod.modURI, err)
+					continue
+				}
+				parsed, err := s.ParseMod(ctx, fh)
+				if err != nil {
+					addError(mod.modURI, err)
+					continue
+				}
+				path := parsed.File.Module.Mod.Path
+				scopes = append(scopes, moduleLoadScope(path))
+			}
+		}
+		if len(scopes) == 0 {
+			scopes = append(scopes, viewLoadScope("LOAD_VIEW"))
+		}
+		err := s.load(ctx, append(scopes, packagePath("builtin"))...)
 		if ctx.Err() != nil {
 			return
 		}
 		if err != nil {
 			event.Error(ctx, "initial workspace load failed", err)
+			if modErrors != nil {
+				s.view.initializedErr = errors.Errorf("errors loading modules: %v: %w", err, modErrors)
+			} else {
+				s.view.initializedErr = err
+			}
 		}
-		v.initializedErr = err
 	})
 }
 
-// AwaitInitialized waits until a view is initialized
-func (v *View) AwaitInitialized(ctx context.Context) {
-	select {
-	case <-ctx.Done():
-		return
-	case <-v.initialized:
-	}
-	// We typically prefer to run something as intensive as the IWL without
-	// blocking. I'm not sure if there is a way to do that here.
-	v.initialize(ctx, v.getSnapshot(), false)
-}
-
 // invalidateContent invalidates the content of a Go file,
 // including any position and type information that depends on it.
 // It returns true if we were already tracking the given file, false otherwise.
@@ -699,7 +787,7 @@ func (v *View) invalidateContent(ctx context.Context, uris map[span.URI]source.V
 	v.cancelBackground()
 
 	// Do not clone a snapshot until its view has finished initializing.
-	v.AwaitInitialized(ctx)
+	v.snapshot.AwaitInitialized(ctx)
 
 	// This should be the only time we hold the view's snapshot lock for any period of time.
 	v.snapshotMu.Lock()
@@ -724,55 +812,93 @@ func (v *View) cancelBackground() {
 }
 
 func (v *View) maybeReinitialize() {
+	v.reinitialize(false)
+}
+
+func (v *View) definitelyReinitialize() {
+	v.reinitialize(true)
+}
+
+func (v *View) reinitialize(force bool) {
 	v.initializationSema <- struct{}{}
 	defer func() {
 		<-v.initializationSema
 	}()
 
-	if v.initializedErr == nil {
+	if !force && v.initializedErr == nil {
 		return
 	}
 	var once sync.Once
 	v.initializeOnce = &once
 }
 
-func (v *View) setBuildInformation(ctx context.Context, folder span.URI, options source.Options) error {
+func (s *Session) getWorkspaceInformation(ctx context.Context, folder span.URI, options *source.Options) (*workspaceInformation, error) {
 	if err := checkPathCase(folder.Filename()); err != nil {
-		return errors.Errorf("invalid workspace configuration: %w", err)
+		return nil, errors.Errorf("invalid workspace configuration: %w", err)
 	}
-	// Make sure to get the `go env` before continuing with initialization.
-	modFile, err := v.setGoEnv(ctx, options.Env)
+	var err error
+	goversion, err := s.goVersion(ctx, folder.Filename(), options.Env)
 	if err != nil {
-		return err
+		return nil, err
 	}
-	if modFile == os.DevNull {
-		return nil
+
+	go111module := os.Getenv("GO111MODULE")
+	for _, kv := range options.Env {
+		split := strings.SplitN(kv, "=", 2)
+		if len(split) != 2 {
+			continue
+		}
+		if split[0] == "GO111MODULE" {
+			go111module = split[1]
+		}
+	}
+	// If using 1.16, change the default back to auto. The primary effect of
+	// GO111MODULE=on is to break GOPATH, which we aren't too interested in.
+	if goversion >= 16 && go111module == "" {
+		go111module = "auto"
+	}
+
+	// Make sure to get the `go env` before continuing with initialization.
+	envVars, env, err := s.getGoEnv(ctx, folder.Filename(), append(options.Env, "GO111MODULE="+go111module))
+	if err != nil {
+		return nil, err
+	}
+	// The value of GOPACKAGESDRIVER is not returned through the go command.
+	gopackagesdriver := os.Getenv("GOPACKAGESDRIVER")
+	for _, s := range env {
+		split := strings.SplitN(s, "=", 2)
+		if split[0] == "GOPACKAGESDRIVER" {
+			gopackagesdriver = split[1]
+		}
+	}
+	// A user may also have a gopackagesdriver binary on their machine, which
+	// works the same way as setting GOPACKAGESDRIVER.
+	tool, _ := exec.LookPath("gopackagesdriver")
+	hasGopackagesDriver := gopackagesdriver != "off" && (gopackagesdriver != "" || tool != "")
+
+	var modURI, sumURI span.URI
+	if envVars.gomod != os.DevNull && envVars.gomod != "" {
+		modURI = span.URIFromPath(envVars.gomod)
 	}
-	v.modURI = span.URIFromPath(modFile)
 	// Set the sumURI, if the go.sum exists.
-	sumFilename := filepath.Join(filepath.Dir(modFile), "go.sum")
+	sumFilename := filepath.Join(filepath.Dir(envVars.gomod), "go.sum")
 	if stat, _ := os.Stat(sumFilename); stat != nil {
-		v.sumURI = span.URIFromPath(sumFilename)
+		sumURI = span.URIFromPath(sumFilename)
 	}
-
-	if options.ExpandWorkspaceToModule && v.modURI != "" {
-		v.root = span.URIFromPath(filepath.Dir(v.modURI.Filename()))
+	root := folder
+	if options.ExpandWorkspaceToModule && modURI != "" {
+		root = span.URIFromPath(filepath.Dir(modURI.Filename()))
 	}
-
-	// Now that we have set all required fields,
-	// check if the view has a valid build configuration.
-	v.setBuildConfiguration()
-
-	// The user has disabled the use of the -modfile flag or has no go.mod file.
-	if !options.TempModfile || v.modURI == "" {
-		return nil
-	}
-	if modfileFlag, err := v.modfileFlagExists(ctx, v.Options().Env); err != nil {
-		return err
-	} else if modfileFlag {
-		v.tmpMod = true
-	}
-	return nil
+	return &workspaceInformation{
+		hasGopackagesDriver:  hasGopackagesDriver,
+		go111module:          go111module,
+		goversion:            goversion,
+		rootURI:              root,
+		environmentVariables: envVars,
+		goEnv:                env,
+		modURI:               modURI,
+		sumURI:               sumURI,
+	}, nil
 }
 
 // OS-specific path case check, for case-insensitive filesystems.
@@ -782,23 +908,24 @@ func defaultCheckPathCase(path string) error {
 	return nil
 }
 
-func (v *View) setBuildConfiguration() (isValid bool) {
-	defer func() {
-		v.hasValidBuildConfiguration = isValid
-	}()
+func validBuildConfiguration(folder span.URI, ws *workspaceInformation, modules map[span.URI]*moduleRoot) bool {
 	// Since we only really understand the `go` command, if the user has a
 	// different GOPACKAGESDRIVER, assume that their configuration is valid.
-	if v.hasGopackagesDriver {
+	if ws.hasGopackagesDriver {
 		return true
 	}
-	// Check if the user is working within a module.
-	if v.modURI != "" {
+	// Check if the user is working within a module or if we have found
+	// multiple modules in the workspace.
+	if ws.modURI != "" {
+		return true
+	}
+	if len(modules) > 0 {
 		return true
 	}
 	// The user may have a multiple directories in their GOPATH.
 	// Check if the workspace is within any of them.
-	for _, gp := range filepath.SplitList(v.gopath) {
-		if isSubdirectory(filepath.Join(gp, "src"), v.folder.Filename()) {
+	for _, gp := range filepath.SplitList(ws.gopath) {
+		if isSubdirectory(filepath.Join(gp, "src"), folder.Filename()) {
 			return true
 		}
 	}
@@ -810,16 +937,15 @@ func isSubdirectory(root, leaf string) bool {
 	return err == nil && !strings.HasPrefix(rel, "..")
 }
 
-// setGoEnv sets the view's various GO* values. It also returns the view's
-// GOMOD value, which need not be cached.
-func (v *View) setGoEnv(ctx context.Context, configEnv []string) (string, error) {
-	var gomod string
+// getGoEnv gets the view's various GO* values.
+func (s *Session) getGoEnv(ctx context.Context, folder string, configEnv []string) (environmentVariables, map[string]string, error) {
+	envVars := environmentVariables{}
 	vars := map[string]*string{
-		"GOCACHE":    &v.gocache,
-		"GOPATH":     &v.gopath,
-		"GOPRIVATE":  &v.goprivate,
-		"GOMODCACHE": &v.gomodcache,
-		"GOMOD":      &gomod,
+		"GOCACHE":    &envVars.gocache,
+		"GOPATH":     &envVars.gopath,
+		"GOPRIVATE":  &envVars.goprivate,
+		"GOMODCACHE": &envVars.gomodcache,
+		"GOMOD":      &envVars.gomod,
 	}
 	// We can save ~200 ms by requesting only the variables we care about.
 	args := append([]string{"-json"}, imports.RequiredGoEnvVars...)
@@ -831,40 +957,28 @@ func (v *View) setGoEnv(ctx context.Context, configEnv []string) (string, error)
 		Verb:       "env",
 		Args:       args,
 		Env:        configEnv,
-		WorkingDir: v.Folder().Filename(),
+		WorkingDir: folder,
 	}
 	// Don't go through runGoCommand, as we don't need a temporary -modfile to
 	// run `go env`.
-	stdout, err := v.session.gocmdRunner.Run(ctx, inv)
+	stdout, err := s.gocmdRunner.Run(ctx, inv)
 	if err != nil {
-		return "", err
+		return environmentVariables{}, nil, err
 	}
-	if err := json.Unmarshal(stdout.Bytes(), &v.goEnv); err != nil {
-		return "", err
+	env := make(map[string]string)
+	if err := json.Unmarshal(stdout.Bytes(), &env); err != nil {
+		return environmentVariables{}, nil, err
 	}
 
 	for key, ptr := range vars {
-		*ptr = v.goEnv[key]
+		*ptr = env[key]
 	}
 
 	// Old versions of Go don't have GOMODCACHE, so emulate it.
-	if v.gomodcache == "" && v.gopath != "" {
-		v.gomodcache = filepath.Join(filepath.SplitList(v.gopath)[0], "pkg/mod")
+	if envVars.gomodcache == "" && envVars.gopath != "" {
+		envVars.gomodcache = filepath.Join(filepath.SplitList(envVars.gopath)[0], "pkg/mod")
 	}
-
-	// The value of GOPACKAGESDRIVER is not returned through the go command.
-	gopackagesdriver := os.Getenv("GOPACKAGESDRIVER")
-	for _, s := range configEnv {
-		split := strings.SplitN(s, "=", 2)
-		if split[0] == "GOPACKAGESDRIVER" {
-			gopackagesdriver = split[1]
-		}
-	}
-	// A user may also have a gopackagesdriver binary on their machine, which
-	// works the same way as setting GOPACKAGESDRIVER.
-	tool, _ := exec.LookPath("gopackagesdriver")
-	v.hasGopackagesDriver = gopackagesdriver != "off" && (gopackagesdriver != "" || tool != "")
-	return gomod, nil
+	return envVars, env, err
 }
 
 func (v *View) IsGoPrivatePath(target string) bool {
@@ -913,28 +1027,87 @@ func globsMatchPath(globs, target string) bool {
 	return false
 }
 
-// This function will return the main go.mod file for this folder if it exists
-// and whether the -modfile flag exists for this version of go.
-func (v *View) modfileFlagExists(ctx context.Context, env []string) (bool, error) {
-	// Check the go version by running "go list" with modules off.
-	// Borrowed from internal/imports/mod.go:620.
-	const format = `{{range context.ReleaseTags}}{{if eq . "go1.14"}}{{.}}{{end}}{{end}}`
-	folder := v.folder.Filename()
-	inv := gocommand.Invocation{
-		Verb:       "list",
-		Args:       []string{"-e", "-f", format},
-		Env:        append(env, "GO111MODULE=off"),
-		WorkingDir: v.root.Filename(),
+var modFlagRegexp = regexp.MustCompile(`-mod[ =](\w+)`)
+
+func (v *View) needsModEqualsMod(ctx context.Context, modFH source.FileHandle) (bool, error) {
+	if v.goversion < 16 || v.workspaceMode&moduleMode == 0 {
+		return false, nil
 	}
-	stdout, err := v.session.gocmdRunner.Run(ctx, inv)
+
+	matches := modFlagRegexp.FindStringSubmatch(v.goEnv["GOFLAGS"])
+	var modFlag string
+	if len(matches) != 0 {
+		modFlag = matches[1]
+	}
+	if modFlag != "" {
+		// Don't override an explicit '-mod=vendor' argument.
+		// We do want to override '-mod=readonly': it would break various module code lenses,
+		// and on 1.16 we know -modfile is available, so we won't mess with go.mod anyway.
+		return modFlag == "vendor", nil
+	}
+
+	// In workspace module mode, there may not be a go.mod file.
+	// TODO: Once vendor mode is designed, update to check if it's on, however that works.
+	if modFH == nil {
+		return true, nil
+	}
+
+	modBytes, err := modFH.Read()
 	if err != nil {
 		return false, err
 	}
-	// If the output is not go1.14 or an empty string, then it could be an error.
-	lines := strings.Split(stdout.String(), "\n")
-	if len(lines) < 2 && stdout.String() != "" {
-		event.Error(ctx, "unexpected stdout when checking for go1.14", errors.Errorf("%q", stdout), tag.Directory.Of(folder))
-		return false, nil
+	modFile, err := modfile.Parse(modFH.URI().Filename(), modBytes, nil)
+	if err != nil {
+		return false, err
 	}
-	return lines[0] == "go1.14", nil
+	if fi, err := os.Stat(filepath.Join(filepath.Dir(v.modURI.Filename()), "vendor")); err != nil || !fi.IsDir() {
+		return true, nil
+	}
+	vendorEnabled := modFile.Go.Version != "" && semver.Compare("v"+modFile.Go.Version, "v1.14") >= 0
+	return !vendorEnabled, nil
+}
+
+// determineWorkspaceMode determines the workspace mode for the given view.
+func determineWorkspaceMode(options *source.Options, validBuildConfiguration bool, ws *workspaceInformation, modules map[span.URI]*moduleRoot) workspaceMode {
+	var mode workspaceMode
+
+	// If the view has an invalid configuration, don't build the workspace
+	// module.
+	if !validBuildConfiguration {
+		return mode
+	}
+	// If the view is not in a module and contains no modules, but still has a
+	// valid workspace configuration, do not create the workspace module.
+	// It could be using GOPATH or a different build system entirely.
+	if ws.modURI == "" && len(modules) == 0 && validBuildConfiguration {
+		return mode
+	}
+	// Check if we should be using module mode.
+	if ws.modURI != "" || len(modules) > 0 {
+		mode |= moduleMode
+	}
+	// The -modfile flag is available for Go versions >= 1.14.
+	if ws.modURI != "" && options.TempModfile && ws.goversion >= 14 {
+		mode |= tempModfile
+	}
+	// Don't default to multi-workspace mode if one of the modules contains a
+	// vendor directory. We still have to decide how to handle vendoring.
+	for _, mod := range modules {
+		if info, _ := os.Stat(filepath.Join(mod.rootURI.Filename(), "vendor")); info != nil {
+			return mode
+		}
+	}
+	// If the user is intentionally limiting their workspace scope, don't
+	// enable multi-module workspace mode.
+	// TODO(rstambler): This should only change the calculation of the root,
+	// not the mode.
+	if !options.ExpandWorkspaceToModule {
+		return mode
+	}
+	// The workspace module has been disabled by the user.
+	if !options.ExperimentalWorkspaceModule {
+		return mode
+	}
+	mode |= usesWorkspaceModule
+	return mode
 }
