@@ -10,8 +10,6 @@ import (
 	"fmt"
 	"io"
 	"strings"
-
-	"golang.org/x/tools/internal/gocommand"
 )
 
 type PrintMode int
@@ -23,7 +21,7 @@ const (
 )
 
 // Version is a manually-updated mechanism for tracking versions.
-var Version = "v0.5.0"
+var Version = "v0.5.1"
 
 // PrintServerInfo writes HTML debug info to w for the Instance.
 func (i *Instance) PrintServerInfo(ctx context.Context, w io.Writer) {
@@ -48,17 +46,6 @@ func PrintVersionInfo(ctx context.Context, w io.Writer, verbose bool, mode Print
 	section(w, mode, "Build info", func() {
 		printBuildInfo(w, true, mode)
 	})
-	fmt.Fprint(w, "\n")
-	section(w, mode, "Go info", func() {
-		gocmdRunner := &gocommand.Runner{}
-		version, err := gocmdRunner.Run(ctx, gocommand.Invocation{
-			Verb: "version",
-		})
-		if err != nil {
-			panic(err)
-		}
-		fmt.Fprintln(w, version.String())
-	})
 }
 
 func section(w io.Writer, mode PrintMode, title string, body func()) {
