@@ -7,7 +7,6 @@ package fake
 import (
 	"bufio"
 	"context"
-	"errors"
 	"fmt"
 	"path/filepath"
 	"regexp"
@@ -18,6 +17,7 @@ import (
 	"golang.org/x/tools/internal/lsp/protocol"
 	"golang.org/x/tools/internal/lsp/source"
 	"golang.org/x/tools/internal/span"
+	errors "golang.org/x/xerrors"
 )
 
 // Editor is a fake editor client.  It keeps track of client state and can be
@@ -67,7 +67,11 @@ type EditorConfig struct {
 
 	// SymbolMatcher is the config associated with the "symbolMatcher" gopls
 	// config option.
-	SymbolMatcher *string
+	SymbolMatcher, SymbolStyle *string
+
+	// LimitWorkspaceScope is true if the user does not want to expand their
+	// workspace scope to the entire module.
+	LimitWorkspaceScope bool
 
 	// WithoutWorkspaceFolders is used to simulate opening a single file in the
 	// editor, without a workspace root. In that case, the client sends neither
@@ -77,6 +81,9 @@ type EditorConfig struct {
 	// EditorRootPath specifies the root path of the workspace folder used when
 	// initializing gopls in the sandbox. If empty, the Workdir is used.
 	EditorRootPath string
+
+	// EnableStaticcheck enables staticcheck analyzers.
+	EnableStaticcheck bool
 }
 
 // NewEditor Creates a new Editor.
@@ -114,7 +121,7 @@ func (e *Editor) Connect(ctx context.Context, conn jsonrpc2.Conn, hooks ClientHo
 func (e *Editor) Shutdown(ctx context.Context) error {
 	if e.Server != nil {
 		if err := e.Server.Shutdown(ctx); err != nil {
-			return fmt.Errorf("Shutdown: %w", err)
+			return errors.Errorf("Shutdown: %w", err)
 		}
 	}
 	return nil
@@ -126,7 +133,7 @@ func (e *Editor) Exit(ctx context.Context) error {
 		// Not all LSP clients issue the exit RPC, but we do so here to ensure that
 		// we gracefully handle it on multi-session servers.
 		if err := e.Server.Exit(ctx); err != nil {
-			return fmt.Errorf("Exit: %w", err)
+			return errors.Errorf("Exit: %w", err)
 		}
 	}
 	return nil
@@ -146,7 +153,7 @@ func (e *Editor) Close(ctx context.Context) error {
 		// connection closed itself
 		return nil
 	case <-ctx.Done():
-		return fmt.Errorf("connection not closed: %w", ctx.Err())
+		return errors.Errorf("connection not closed: %w", ctx.Err())
 	}
 }
 
@@ -170,15 +177,24 @@ func (e *Editor) configuration() map[string]interface{} {
 	config := map[string]interface{}{
 		"verboseWorkDoneProgress": true,
 		"env":                     e.overlayEnv(),
+		"expandWorkspaceToModule": !e.Config.LimitWorkspaceScope,
+		"completionBudget":        "10s",
 	}
 
 	if e.Config.CodeLens != nil {
 		config["codelens"] = e.Config.CodeLens
 	}
-
 	if e.Config.SymbolMatcher != nil {
 		config["symbolMatcher"] = *e.Config.SymbolMatcher
 	}
+	if e.Config.SymbolStyle != nil {
+		config["symbolStyle"] = *e.Config.SymbolStyle
+	}
+	if e.Config.EnableStaticcheck {
+		config["staticcheck"] = true
+	}
+	// Default to using the experimental workspace module mode.
+	config["experimentalWorkspaceModule"] = true
 
 	return config
 }
@@ -202,19 +218,25 @@ func (e *Editor) initialize(ctx context.Context, withoutWorkspaceFolders bool, e
 	// TODO: set client capabilities
 	params.InitializationOptions = e.configuration()
 
+	// This is a bit of a hack, since the fake editor doesn't actually support
+	// watching changed files that match a specific glob pattern. However, the
+	// editor does send didChangeWatchedFiles notifications, so set this to
+	// true.
+	params.Capabilities.Workspace.DidChangeWatchedFiles.DynamicRegistration = true
+
 	params.Trace = "messages"
 	// TODO: support workspace folders.
 	if e.Server != nil {
 		resp, err := e.Server.Initialize(ctx, params)
 		if err != nil {
-			return fmt.Errorf("initialize: %w", err)
+			return errors.Errorf("initialize: %w", err)
 		}
 		e.mu.Lock()
 		e.serverCapabilities = resp.Capabilities
 		e.mu.Unlock()
 
 		if err := e.Server.Initialized(ctx, &protocol.InitializedParams{}); err != nil {
-			return fmt.Errorf("initialized: %w", err)
+			return errors.Errorf("initialized: %w", err)
 		}
 	}
 	// TODO: await initial configuration here, or expect gopls to manage that?
@@ -256,7 +278,7 @@ func (e *Editor) OpenFileWithContent(ctx context.Context, path, content string)
 		if err := e.Server.DidOpen(ctx, &protocol.DidOpenTextDocumentParams{
 			TextDocument: item,
 		}); err != nil {
-			return fmt.Errorf("DidOpen: %w", err)
+			return errors.Errorf("DidOpen: %w", err)
 		}
 	}
 	return nil
@@ -298,7 +320,7 @@ func (e *Editor) CreateBuffer(ctx context.Context, path, content string) error {
 		if err := e.Server.DidOpen(ctx, &protocol.DidOpenTextDocumentParams{
 			TextDocument: item,
 		}); err != nil {
-			return fmt.Errorf("DidOpen: %w", err)
+			return errors.Errorf("DidOpen: %w", err)
 		}
 	}
 	return nil
@@ -319,7 +341,7 @@ func (e *Editor) CloseBuffer(ctx context.Context, path string) error {
 		if err := e.Server.DidClose(ctx, &protocol.DidCloseTextDocumentParams{
 			TextDocument: e.textDocumentIdentifier(path),
 		}); err != nil {
-			return fmt.Errorf("DidClose: %w", err)
+			return errors.Errorf("DidClose: %w", err)
 		}
 	}
 	return nil
@@ -335,10 +357,10 @@ func (e *Editor) textDocumentIdentifier(path string) protocol.TextDocumentIdenti
 // the filesystem.
 func (e *Editor) SaveBuffer(ctx context.Context, path string) error {
 	if err := e.OrganizeImports(ctx, path); err != nil {
-		return fmt.Errorf("organizing imports before save: %w", err)
+		return errors.Errorf("organizing imports before save: %w", err)
 	}
 	if err := e.FormatBuffer(ctx, path); err != nil {
-		return fmt.Errorf("formatting before save: %w", err)
+		return errors.Errorf("formatting before save: %w", err)
 	}
 	return e.SaveBufferWithoutActions(ctx, path)
 }
@@ -364,11 +386,11 @@ func (e *Editor) SaveBufferWithoutActions(ctx context.Context, path string) erro
 			TextDocument: docID,
 			Reason:       protocol.Manual,
 		}); err != nil {
-			return fmt.Errorf("WillSave: %w", err)
+			return errors.Errorf("WillSave: %w", err)
 		}
 	}
 	if err := e.sandbox.Workdir.WriteFile(ctx, path, content); err != nil {
-		return fmt.Errorf("writing %q: %w", path, err)
+		return errors.Errorf("writing %q: %w", path, err)
 	}
 	if e.Server != nil {
 		params := &protocol.DidSaveTextDocumentParams{
@@ -381,7 +403,7 @@ func (e *Editor) SaveBufferWithoutActions(ctx context.Context, path string) erro
 			params.Text = &content
 		}
 		if err := e.Server.DidSave(ctx, params); err != nil {
-			return fmt.Errorf("DidSave: %w", err)
+			return errors.Errorf("DidSave: %w", err)
 		}
 	}
 	return nil
@@ -402,10 +424,10 @@ func contentPosition(content string, offset int) (Pos, error) {
 		line++
 	}
 	if err := scanner.Err(); err != nil {
-		return Pos{}, fmt.Errorf("scanning content: %w", err)
+		return Pos{}, errors.Errorf("scanning content: %w", err)
 	}
 	// Scan() will drop the last line if it is empty. Correct for this.
-	if strings.HasSuffix(content, "\n") && offset == start {
+	if (strings.HasSuffix(content, "\n") || content == "") && offset == start {
 		return Pos{Line: line, Column: 0}, nil
 	}
 	return Pos{}, fmt.Errorf("position %d out of bounds in %q (line = %d, start = %d)", offset, content, line, start)
@@ -450,6 +472,18 @@ func regexpRange(content, re string) (Pos, Pos, error) {
 	return startPos, endPos, nil
 }
 
+// RegexpRange returns the first range in the buffer bufName matching re. See
+// RegexpSearch for more information on matching.
+func (e *Editor) RegexpRange(bufName, re string) (Pos, Pos, error) {
+	e.mu.Lock()
+	defer e.mu.Unlock()
+	buf, ok := e.buffers[bufName]
+	if !ok {
+		return Pos{}, Pos{}, ErrUnknownBuffer
+	}
+	return regexpRange(buf.text(), re)
+}
+
 // RegexpSearch returns the position of the first match for re in the buffer
 // bufName. For convenience, RegexpSearch supports the following two modes:
 //  1. If re has no subgroups, return the position of the match for re itself.
@@ -457,13 +491,7 @@ func regexpRange(content, re string) (Pos, Pos, error) {
 // It returns an error re is invalid, has more than one subgroup, or doesn't
 // match the buffer.
 func (e *Editor) RegexpSearch(bufName, re string) (Pos, error) {
-	e.mu.Lock()
-	defer e.mu.Unlock()
-	buf, ok := e.buffers[bufName]
-	if !ok {
-		return Pos{}, ErrUnknownBuffer
-	}
-	start, _, err := regexpRange(buf.text(), re)
+	start, _, err := e.RegexpRange(bufName, re)
 	return start, err
 }
 
@@ -550,7 +578,7 @@ func (e *Editor) editBufferLocked(ctx context.Context, path string, edits []Edit
 	}
 	if e.Server != nil {
 		if err := e.Server.DidChange(ctx, params); err != nil {
-			return fmt.Errorf("DidChange: %w", err)
+			return errors.Errorf("DidChange: %w", err)
 		}
 	}
 	return nil
@@ -564,11 +592,11 @@ func (e *Editor) GoToDefinition(ctx context.Context, path string, pos Pos) (stri
 	}
 	params := &protocol.DefinitionParams{}
 	params.TextDocument.URI = e.sandbox.Workdir.URI(path)
-	params.Position = pos.toProtocolPosition()
+	params.Position = pos.ToProtocolPosition()
 
 	resp, err := e.Server.Definition(ctx, params)
 	if err != nil {
-		return "", Pos{}, fmt.Errorf("definition: %w", err)
+		return "", Pos{}, errors.Errorf("definition: %w", err)
 	}
 	if len(resp) == 0 {
 		return "", Pos{}, nil
@@ -576,7 +604,7 @@ func (e *Editor) GoToDefinition(ctx context.Context, path string, pos Pos) (stri
 	newPath := e.sandbox.Workdir.URIToPath(resp[0].URI)
 	newPos := fromProtocolPosition(resp[0].Range.Start)
 	if err := e.OpenFile(ctx, newPath); err != nil {
-		return "", Pos{}, fmt.Errorf("OpenFile: %w", err)
+		return "", Pos{}, errors.Errorf("OpenFile: %w", err)
 	}
 	return newPath, newPos, nil
 }
@@ -588,7 +616,7 @@ func (e *Editor) Symbol(ctx context.Context, query string) ([]SymbolInformation,
 
 	resp, err := e.Server.Symbol(ctx, params)
 	if err != nil {
-		return nil, fmt.Errorf("symbol: %w", err)
+		return nil, errors.Errorf("symbol: %w", err)
 	}
 	var res []SymbolInformation
 	for _, si := range resp {
@@ -643,7 +671,7 @@ func (e *Editor) codeAction(ctx context.Context, path string, rng *protocol.Rang
 	}
 	actions, err := e.Server.CodeAction(ctx, params)
 	if err != nil {
-		return fmt.Errorf("textDocument/codeAction: %w", err)
+		return errors.Errorf("textDocument/codeAction: %w", err)
 	}
 	for _, action := range actions {
 		var match bool
@@ -664,7 +692,7 @@ func (e *Editor) codeAction(ctx context.Context, path string, rng *protocol.Rang
 			}
 			edits := convertEdits(change.Edits)
 			if err := e.EditBuffer(ctx, path, edits); err != nil {
-				return fmt.Errorf("editing buffer %q: %w", path, err)
+				return errors.Errorf("editing buffer %q: %w", path, err)
 			}
 		}
 		// Execute any commands. The specification says that commands are
@@ -701,7 +729,7 @@ func (e *Editor) FormatBuffer(ctx context.Context, path string) error {
 	params.TextDocument.URI = e.sandbox.Workdir.URI(path)
 	resp, err := e.Server.Formatting(ctx, params)
 	if err != nil {
-		return fmt.Errorf("textDocument/formatting: %w", err)
+		return errors.Errorf("textDocument/formatting: %w", err)
 	}
 	e.mu.Lock()
 	defer e.mu.Unlock()
@@ -773,6 +801,30 @@ func (e *Editor) CodeLens(ctx context.Context, path string) ([]protocol.CodeLens
 	return lens, nil
 }
 
+// Completion executes a completion request on the server.
+func (e *Editor) Completion(ctx context.Context, path string, pos Pos) (*protocol.CompletionList, error) {
+	if e.Server == nil {
+		return nil, nil
+	}
+	e.mu.Lock()
+	_, ok := e.buffers[path]
+	e.mu.Unlock()
+	if !ok {
+		return nil, fmt.Errorf("buffer %q is not open", path)
+	}
+	params := &protocol.CompletionParams{
+		TextDocumentPositionParams: protocol.TextDocumentPositionParams{
+			TextDocument: e.textDocumentIdentifier(path),
+			Position:     pos.ToProtocolPosition(),
+		},
+	}
+	completions, err := e.Server.Completion(ctx, params)
+	if err != nil {
+		return nil, err
+	}
+	return completions, nil
+}
+
 // References executes a reference request on the server.
 func (e *Editor) References(ctx context.Context, path string, pos Pos) ([]protocol.Location, error) {
 	if e.Server == nil {
@@ -787,7 +839,7 @@ func (e *Editor) References(ctx context.Context, path string, pos Pos) ([]protoc
 	params := &protocol.ReferenceParams{
 		TextDocumentPositionParams: protocol.TextDocumentPositionParams{
 			TextDocument: e.textDocumentIdentifier(path),
-			Position:     pos.toProtocolPosition(),
+			Position:     pos.ToProtocolPosition(),
 		},
 		Context: protocol.ReferenceContext{
 			IncludeDeclaration: true,
@@ -831,11 +883,11 @@ func (e *Editor) Hover(ctx context.Context, path string, pos Pos) (*protocol.Mar
 	}
 	params := &protocol.HoverParams{}
 	params.TextDocument.URI = e.sandbox.Workdir.URI(path)
-	params.Position = pos.toProtocolPosition()
+	params.Position = pos.ToProtocolPosition()
 
 	resp, err := e.Server.Hover(ctx, params)
 	if err != nil {
-		return nil, Pos{}, fmt.Errorf("hover: %w", err)
+		return nil, Pos{}, errors.Errorf("hover: %w", err)
 	}
 	if resp == nil {
 		return nil, Pos{}, nil
