@@ -7,10 +7,12 @@ package source
 import (
 	"context"
 	"fmt"
-	"os"
 	"regexp"
+	"strings"
+	"sync"
 	"time"
 
+	"golang.org/x/tools/go/analysis"
 	"golang.org/x/tools/go/analysis/passes/asmdecl"
 	"golang.org/x/tools/go/analysis/passes/assign"
 	"golang.org/x/tools/go/analysis/passes/atomic"
@@ -52,77 +54,88 @@ import (
 	errors "golang.org/x/xerrors"
 )
 
+var (
+	optionsOnce    sync.Once
+	defaultOptions *Options
+)
+
+//go:generate go run golang.org/x/tools/internal/lsp/source/genopts -output options_json.go
+
 // DefaultOptions is the options that are used for Gopls execution independent
 // of any externally provided configuration (LSP initialization, command
 // invokation, etc.).
-func DefaultOptions() Options {
-	var commands []string
-	for _, c := range Commands {
-		commands = append(commands, c.Name)
-	}
-	return Options{
-		ClientOptions: ClientOptions{
-			InsertTextFormat:                  protocol.PlainTextTextFormat,
-			PreferredContentFormat:            protocol.Markdown,
-			ConfigurationSupported:            true,
-			DynamicConfigurationSupported:     true,
-			DynamicWatchedFilesSupported:      true,
-			LineFoldingOnly:                   false,
-			HierarchicalDocumentSymbolSupport: true,
-		},
-		ServerOptions: ServerOptions{
-			SupportedCodeActions: map[FileKind]map[protocol.CodeActionKind]bool{
-				Go: {
-					protocol.SourceFixAll:          true,
-					protocol.SourceOrganizeImports: true,
-					protocol.QuickFix:              true,
-					protocol.RefactorRewrite:       true,
-					protocol.RefactorExtract:       true,
-				},
-				Mod: {
-					protocol.SourceOrganizeImports: true,
-				},
-				Sum: {},
+func DefaultOptions() *Options {
+	optionsOnce.Do(func() {
+		var commands []string
+		for _, c := range Commands {
+			commands = append(commands, c.Name)
+		}
+		defaultOptions = &Options{
+			ClientOptions: ClientOptions{
+				InsertTextFormat:                  protocol.PlainTextTextFormat,
+				PreferredContentFormat:            protocol.Markdown,
+				ConfigurationSupported:            true,
+				DynamicConfigurationSupported:     true,
+				DynamicWatchedFilesSupported:      true,
+				LineFoldingOnly:                   false,
+				HierarchicalDocumentSymbolSupport: true,
 			},
-			SupportedCommands: commands,
-		},
-		UserOptions: UserOptions{
-			Env:                     os.Environ(),
-			HoverKind:               FullDocumentation,
-			LinkTarget:              "pkg.go.dev",
-			LinksInHover:            true,
-			Matcher:                 Fuzzy,
-			SymbolMatcher:           SymbolFuzzy,
-			DeepCompletion:          true,
-			UnimportedCompletion:    true,
-			CompletionDocumentation: true,
-			EnabledCodeLens: map[string]bool{
-				CommandGenerate.Name:          true,
-				CommandRegenerateCgo.Name:     true,
-				CommandTidy.Name:              true,
-				CommandToggleDetails.Name:     false,
-				CommandUpgradeDependency.Name: true,
-				CommandVendor.Name:            true,
+			ServerOptions: ServerOptions{
+				SupportedCodeActions: map[FileKind]map[protocol.CodeActionKind]bool{
+					Go: {
+						protocol.SourceFixAll:          true,
+						protocol.SourceOrganizeImports: true,
+						protocol.QuickFix:              true,
+						protocol.RefactorRewrite:       true,
+						protocol.RefactorExtract:       true,
+					},
+					Mod: {
+						protocol.SourceOrganizeImports: true,
+					},
+					Sum: {},
+				},
+				SupportedCommands: commands,
 			},
-			ExpandWorkspaceToModule: true,
-		},
-		DebuggingOptions: DebuggingOptions{
-			CompletionBudget:   100 * time.Millisecond,
-			LiteralCompletions: true,
-		},
-		ExperimentalOptions: ExperimentalOptions{
-			TempModfile: true,
-		},
-		Hooks: Hooks{
-			ComputeEdits:         myers.ComputeEdits,
-			URLRegexp:            urlRegexp(),
-			DefaultAnalyzers:     defaultAnalyzers(),
-			TypeErrorAnalyzers:   typeErrorAnalyzers(),
-			ConvenienceAnalyzers: convenienceAnalyzers(),
-			StaticcheckAnalyzers: map[string]Analyzer{},
-			GoDiff:               true,
-		},
-	}
+			UserOptions: UserOptions{
+				HoverKind:  FullDocumentation,
+				LinkTarget: "pkg.go.dev",
+			},
+			DebuggingOptions: DebuggingOptions{
+				CompletionBudget:   100 * time.Millisecond,
+				LiteralCompletions: true,
+			},
+			ExperimentalOptions: ExperimentalOptions{
+				TempModfile:             true,
+				ExpandWorkspaceToModule: true,
+				Codelens: map[string]bool{
+					CommandGenerate.Name:          true,
+					CommandRegenerateCgo.Name:     true,
+					CommandTidy.Name:              true,
+					CommandToggleDetails.Name:     false,
+					CommandUpgradeDependency.Name: true,
+					CommandVendor.Name:            true,
+				},
+				LinksInHover:            true,
+				CompleteUnimported:      true,
+				CompletionDocumentation: true,
+				DeepCompletion:          true,
+				ImportShortcut:          Both,
+				Matcher:                 Fuzzy,
+				SymbolMatcher:           SymbolFuzzy,
+				SymbolStyle:             PackageQualifiedSymbols,
+			},
+			Hooks: Hooks{
+				ComputeEdits:         myers.ComputeEdits,
+				URLRegexp:            urlRegexp(),
+				DefaultAnalyzers:     defaultAnalyzers(),
+				TypeErrorAnalyzers:   typeErrorAnalyzers(),
+				ConvenienceAnalyzers: convenienceAnalyzers(),
+				StaticcheckAnalyzers: map[string]Analyzer{},
+				GoDiff:               true,
+			},
+		}
+	})
+	return defaultOptions
 }
 
 // Options holds various configuration that affects Gopls execution, organized
@@ -158,96 +171,36 @@ type ServerOptions struct {
 // UserOptions holds custom Gopls configuration (not part of the LSP) that is
 // modified by the client.
 type UserOptions struct {
-	// Env is the current set of environment overrides on this view.
-	Env []string
-
-	// BuildFlags is used to adjust the build flags applied to the view.
+	// BuildFlags is the set of flags passed on to the build system when invoked.
+	// It is applied to queries like `go list`, which is used when discovering files.
+	// The most common use is to set `-tags`.
 	BuildFlags []string
 
-	// HoverKind specifies the format of the content for hover requests.
+	// Env adds environment variables to external commands run by `gopls`, most notably `go list`.
+	Env []string
+
+	// HoverKind controls the information that appears in the hover text.
+	// SingleLine and Structured are intended for use only by authors of editor plugins.
 	HoverKind HoverKind
 
-	// UserEnabledAnalyses specifies analyses that the user would like to enable
-	// or disable. A map of the names of analysis passes that should be
-	// enabled/disabled. A full list of analyzers that gopls uses can be found
-	// [here](analyzers.md).
+	// Placeholders enables placeholders for function parameters or struct fields in completion responses.
+	UsePlaceholders bool
+
+	// LinkTarget controls where documentation links go.
+	// It might be one of:
 	//
-	// Example Usage:
-	// ...
-	// "analyses": {
-	//   "unreachable": false, // Disable the unreachable analyzer.
-	//   "unusedparams": true  // Enable the unusedparams analyzer.
-	// }
-	UserEnabledAnalyses map[string]bool
-
-	// EnabledCodeLens specifies which codelens are enabled, keyed by the gopls
-	// command that they provide.
-	EnabledCodeLens map[string]bool
-
-	// StaticCheck enables additional analyses from staticcheck.io.
-	StaticCheck bool
-
-	// LinkTarget is the website used for documentation. If empty, no link is
-	// provided.
+	// * `"godoc.org"`
+	// * `"pkg.go.dev"`
+	//
+	// If company chooses to use its own `godoc.org`, its address can be used as well.
 	LinkTarget string
 
-	// LinksInHover toggles the presence of links to documentation in hover.
-	LinksInHover bool
-
-	// ImportShortcut specifies whether import statements should link to
-	// documentation or go to definitions. The default is both.
-	ImportShortcut ImportShortcut
-
-	// LocalPrefix is used to specify goimports's -local behavior.
-	LocalPrefix string
-
-	// Matcher specifies the type of matcher to use for completion requests.
-	Matcher Matcher
-
-	// SymbolMatcher specifies the type of matcher to use for symbol requests.
-	SymbolMatcher SymbolMatcher
-
-	// SymbolStyle specifies what style of symbols to return in symbol requests
-	// (package qualified, fully qualified, etc).
-	SymbolStyle SymbolStyle
-
-	// DeepCompletion allows completion to perform nested searches through
-	// possible candidates.
-	DeepCompletion bool
-
-	// UnimportedCompletion enables completion for unimported packages.
-	UnimportedCompletion bool
-
-	// CompletionDocumentation returns additional documentation with completion
-	// requests.
-	CompletionDocumentation bool
-
-	// Placeholders adds placeholders to parameters and structs in completion
-	// results.
-	Placeholders bool
+	// Local is the equivalent of the `goimports -local` flag, which puts imports beginning with this string after 3rd-party packages.
+	// It should be the prefix of the import path whose imports should be grouped separately.
+	Local string
 
 	// Gofumpt indicates if we should run gofumpt formatting.
 	Gofumpt bool
-
-	// ExpandWorkspaceToModule is true if we should expand the scope of the
-	// workspace to include the modules containing the workspace folders.
-	ExpandWorkspaceToModule bool
-}
-
-type ImportShortcut int
-
-const (
-	Both ImportShortcut = iota
-	Link
-	Definition
-)
-
-func (s ImportShortcut) ShowLinks() bool {
-	return s == Both || s == Link
-}
-
-func (s ImportShortcut) ShowDefinition() bool {
-	return s == Both || s == Definition
 }
 
 // Hooks contains configuration that is provided to the Gopls command by the
@@ -256,36 +209,129 @@ type Hooks struct {
 	GoDiff               bool
 	ComputeEdits         diff.ComputeEdits
 	URLRegexp            *regexp.Regexp
+	GofumptFormat        func(ctx context.Context, src []byte) ([]byte, error)
 	DefaultAnalyzers     map[string]Analyzer
 	TypeErrorAnalyzers   map[string]Analyzer
 	ConvenienceAnalyzers map[string]Analyzer
 	StaticcheckAnalyzers map[string]Analyzer
-	GofumptFormat        func(ctx context.Context, src []byte) ([]byte, error)
 }
 
 // ExperimentalOptions defines configuration for features under active
 // development. WARNING: This configuration will be changed in the future. It
 // only exists while these features are under development.
 type ExperimentalOptions struct {
+	// Analyses specify analyses that the user would like to enable or disable.
+	// A map of the names of analysis passes that should be enabled/disabled.
+	// A full list of analyzers that gopls uses can be found [here](analyzers.md)
+	//
+	// Example Usage:
+	// ```json5
+	// ...
+	// "analyses": {
+	//   "unreachable": false, // Disable the unreachable analyzer.
+	//   "unusedparams": true  // Enable the unusedparams analyzer.
+	// }
+	// ...
+	// ```
+	Analyses map[string]bool
+
+	// Overrides the enabled/disabled state of various code lenses. Currently, we
+	// support several code lenses:
+	//
+	// * `generate`: run `go generate` as specified by a `//go:generate` directive.
+	// * `upgrade_dependency`: upgrade a dependency listed in a `go.mod` file.
+	// * `test`: run `go test -run` for a test func.
+	// * `gc_details`: Show the gc compiler's choices for inline analysis and escaping.
+	//
+	// Example Usage:
+	// ```json5
+	// "gopls": {
+	// ...
+	//   "codelens": {
+	//     "generate": false,  // Don't run `go generate`.
+	//     "gc_details": true  // Show a code lens toggling the display of gc's choices.
+	//   }
+	// ...
+	// }
+	// ```
+	Codelens map[string]bool
+
+	// CompletionDocumentation enables documentation with completion results.
+	CompletionDocumentation bool
+
+	// CompleteUnimported enables completion for packages that you do not currently import.
+	CompleteUnimported bool
+
+	// DeepCompletion If true, this turns on the ability to return completions from deep inside relevant entities, rather than just the locally accessible ones.
+	//
+	// Consider this example:
+	//
+	// ```go
+	// package main
+	//
+	// import "fmt"
+	//
+	// type wrapString struct {
+	//     str string
+	// }
+	//
+	// func main() {
+	//     x := wrapString{"hello world"}
+	//     fmt.Printf(<>)
+	// }
+	// ```
+	//
+	// At the location of the `<>` in this program, deep completion would suggest the result `x.str`.
+	DeepCompletion bool
+
+	// Matcher sets the algorithm that is used when calculating completion candidates.
+	Matcher Matcher
+
+	// Annotations suppress various kinds of optimization diagnostics
+	// that would be reported by the gc_details command.
+	//  * noNilcheck suppresses display of nilchecks.
+	//  * noEscape suppresses escape choices.
+	//  * noInline suppresses inlining choices.
+	//  * noBounds suppresses bounds checking diagnostics.
+	Annotations map[string]bool
+
+	// Staticcheck enables additional analyses from staticcheck.io.
+	Staticcheck bool
+
+	// SymbolMatcher sets the algorithm that is used when finding workspace symbols.
+	SymbolMatcher SymbolMatcher
+
+	// SymbolStyle specifies what style of symbols to return in symbol requests.
+	SymbolStyle SymbolStyle
+
+	// LinksInHover toggles the presence of links to documentation in hover.
+	LinksInHover bool
+
 	// TempModfile controls the use of the -modfile flag in Go 1.14.
 	TempModfile bool
 
+	// ImportShortcut specifies whether import statements should link to
+	// documentation or go to definitions.
+	ImportShortcut ImportShortcut
+
 	// VerboseWorkDoneProgress controls whether the LSP server should send
 	// progress reports for all work done outside the scope of an RPC.
 	VerboseWorkDoneProgress bool
 
-	// Annotations suppress various kinds of optimization diagnostics
-	// that would be reported by the gc_details command.
-	//   noNilcheck suppresses display of nilchecks.
-	//   noEscape suppresses escape choices.
-	//   noInline suppresses inlining choices.
-	//   noBounds suppresses bounds checking diagnositcs.
-	Annotations map[string]bool
+	// ExpandWorkspaceToModule instructs `gopls` to expand the scope of the workspace to include the
+	// modules containing the workspace folders. Set this to false to avoid loading
+	// your entire module. This is particularly useful for those working in a monorepo.
+	ExpandWorkspaceToModule bool
+
+	// ExperimentalWorkspaceModule opts a user into the experimental support
+	// for multi-module workspaces.
+	ExperimentalWorkspaceModule bool
 }
 
 // DebuggingOptions should not affect the logical execution of Gopls, but may
 // be altered for debugging purposes.
 type DebuggingOptions struct {
+	// VerboseOutput enables additional debug logging.
 	VerboseOutput bool
 
 	// CompletionBudget is the soft latency goal for completion requests. Most
@@ -301,44 +347,60 @@ type DebuggingOptions struct {
 	LiteralCompletions bool
 }
 
-type Matcher int
+type ImportShortcut string
 
 const (
-	Fuzzy = Matcher(iota)
-	CaseInsensitive
-	CaseSensitive
+	Both       ImportShortcut = "Both"
+	Link       ImportShortcut = "Link"
+	Definition ImportShortcut = "Definition"
 )
 
-type SymbolMatcher int
+func (s ImportShortcut) ShowLinks() bool {
+	return s == Both || s == Link
+}
+
+func (s ImportShortcut) ShowDefinition() bool {
+	return s == Both || s == Definition
+}
+
+type Matcher string
 
 const (
-	SymbolFuzzy = SymbolMatcher(iota)
-	SymbolCaseInsensitive
-	SymbolCaseSensitive
+	Fuzzy           Matcher = "Fuzzy"
+	CaseInsensitive Matcher = "CaseInsensitive"
+	CaseSensitive   Matcher = "CaseSensitive"
 )
 
-type SymbolStyle int
+type SymbolMatcher string
 
 const (
-	PackageQualifiedSymbols = SymbolStyle(iota)
-	FullyQualifiedSymbols
-	DynamicSymbols
+	SymbolFuzzy           SymbolMatcher = "Fuzzy"
+	SymbolCaseInsensitive SymbolMatcher = "CaseInsensitive"
+	SymbolCaseSensitive   SymbolMatcher = "CaseSensitive"
 )
 
-type HoverKind int
+type SymbolStyle string
 
 const (
-	SingleLine = HoverKind(iota)
-	NoDocumentation
-	SynopsisDocumentation
-	FullDocumentation
+	PackageQualifiedSymbols SymbolStyle = "Package"
+	FullyQualifiedSymbols   SymbolStyle = "Full"
+	DynamicSymbols          SymbolStyle = "Dynamic"
+)
+
+type HoverKind string
+
+const (
+	SingleLine            HoverKind = "SingleLine"
+	NoDocumentation       HoverKind = "NoDocumentation"
+	SynopsisDocumentation HoverKind = "SynopsisDocumentation"
+	FullDocumentation     HoverKind = "FullDocumentation"
 
 	// Structured is an experimental setting that returns a structured hover format.
 	// This format separates the signature from the documentation, so that the client
 	// can do more manipulation of these fields.
 	//
 	// This should only be used by clients that support this behavior.
-	Structured
+	Structured HoverKind = "Structured"
 )
 
 type OptionResults []OptionResult
@@ -400,6 +462,59 @@ func (o *Options) ForClientCapabilities(caps protocol.ClientCapabilities) {
 	o.HierarchicalDocumentSymbolSupport = caps.TextDocument.DocumentSymbol.HierarchicalDocumentSymbolSupport
 }
 
+func (o *Options) Clone() *Options {
+	result := &Options{
+		ClientOptions:       o.ClientOptions,
+		DebuggingOptions:    o.DebuggingOptions,
+		ExperimentalOptions: o.ExperimentalOptions,
+		Hooks: Hooks{
+			GoDiff:        o.Hooks.GoDiff,
+			ComputeEdits:  o.Hooks.ComputeEdits,
+			GofumptFormat: o.GofumptFormat,
+			URLRegexp:     o.URLRegexp,
+		},
+		ServerOptions: o.ServerOptions,
+		UserOptions:   o.UserOptions,
+	}
+	// Fully clone any slice or map fields. Only Hooks, ExperimentalOptions,
+	// and UserOptions can be modified.
+	copyStringMap := func(src map[string]bool) map[string]bool {
+		dst := make(map[string]bool)
+		for k, v := range src {
+			dst[k] = v
+		}
+		return dst
+	}
+	result.Analyses = copyStringMap(o.Analyses)
+	result.Annotations = copyStringMap(o.Annotations)
+	result.Codelens = copyStringMap(o.Codelens)
+
+	copySlice := func(src []string) []string {
+		dst := make([]string, len(src))
+		copy(dst, src)
+		return dst
+	}
+	result.Env = copySlice(o.Env)
+	result.BuildFlags = copySlice(o.BuildFlags)
+
+	copyAnalyzerMap := func(src map[string]Analyzer) map[string]Analyzer {
+		dst := make(map[string]Analyzer)
+		for k, v := range src {
+			dst[k] = v
+		}
+		return dst
+	}
+	result.DefaultAnalyzers = copyAnalyzerMap(o.DefaultAnalyzers)
+	result.TypeErrorAnalyzers = copyAnalyzerMap(o.TypeErrorAnalyzers)
+	result.ConvenienceAnalyzers = copyAnalyzerMap(o.ConvenienceAnalyzers)
+	result.StaticcheckAnalyzers = copyAnalyzerMap(o.StaticcheckAnalyzers)
+	return result
+}
+
+func (options *Options) AddStaticcheckAnalyzer(a *analysis.Analyzer) {
+	options.StaticcheckAnalyzers[a.Name] = Analyzer{Analyzer: a, Enabled: true}
+}
+
 func (o *Options) set(name string, value interface{}) OptionResult {
 	result := OptionResult{Name: name, Value: value}
 	switch name {
@@ -428,11 +543,11 @@ func (o *Options) set(name string, value interface{}) OptionResult {
 	case "completionDocumentation":
 		result.setBool(&o.CompletionDocumentation)
 	case "usePlaceholders":
-		result.setBool(&o.Placeholders)
+		result.setBool(&o.UsePlaceholders)
 	case "deepCompletion":
 		result.setBool(&o.DeepCompletion)
 	case "completeUnimported":
-		result.setBool(&o.UnimportedCompletion)
+		result.setBool(&o.CompleteUnimported)
 	case "completionBudget":
 		if v, ok := result.asString(); ok {
 			d, err := time.ParseDuration(v)
@@ -448,10 +563,10 @@ func (o *Options) set(name string, value interface{}) OptionResult {
 		if !ok {
 			break
 		}
-		switch matcher {
+		switch strings.ToLower(matcher) {
 		case "fuzzy":
 			o.Matcher = Fuzzy
-		case "caseSensitive":
+		case "casesensitive":
 			o.Matcher = CaseSensitive
 		default:
 			o.Matcher = CaseInsensitive
@@ -462,10 +577,10 @@ func (o *Options) set(name string, value interface{}) OptionResult {
 		if !ok {
 			break
 		}
-		switch matcher {
+		switch strings.ToLower(matcher) {
 		case "fuzzy":
 			o.SymbolMatcher = SymbolFuzzy
-		case "caseSensitive":
+		case "casesensitive":
 			o.SymbolMatcher = SymbolCaseSensitive
 		default:
 			o.SymbolMatcher = SymbolCaseInsensitive
@@ -476,7 +591,7 @@ func (o *Options) set(name string, value interface{}) OptionResult {
 		if !ok {
 			break
 		}
-		switch style {
+		switch strings.ToLower(style) {
 		case "full":
 			o.SymbolStyle = FullyQualifiedSymbols
 		case "dynamic":
@@ -492,16 +607,16 @@ func (o *Options) set(name string, value interface{}) OptionResult {
 		if !ok {
 			break
 		}
-		switch hoverKind {
-		case "NoDocumentation":
+		switch strings.ToLower(hoverKind) {
+		case "nodocumentation":
 			o.HoverKind = NoDocumentation
-		case "SingleLine":
+		case "singleline":
 			o.HoverKind = SingleLine
-		case "SynopsisDocumentation":
+		case "synopsisdocumentation":
 			o.HoverKind = SynopsisDocumentation
-		case "FullDocumentation":
+		case "fulldocumentation":
 			o.HoverKind = FullDocumentation
-		case "Structured":
+		case "structured":
 			o.HoverKind = Structured
 		default:
 			result.errorf("Unsupported hover kind %q", hoverKind)
@@ -516,7 +631,7 @@ func (o *Options) set(name string, value interface{}) OptionResult {
 	case "importShortcut":
 		var s string
 		result.setString(&s)
-		switch s {
+		switch strings.ToLower(s) {
 		case "both":
 			o.ImportShortcut = Both
 		case "link":
@@ -526,7 +641,7 @@ func (o *Options) set(name string, value interface{}) OptionResult {
 		}
 
 	case "analyses":
-		result.setBoolMap(&o.UserEnabledAnalyses)
+		result.setBoolMap(&o.Analyses)
 
 	case "annotations":
 		result.setBoolMap(&o.Annotations)
@@ -544,19 +659,19 @@ func (o *Options) set(name string, value interface{}) OptionResult {
 		var lensOverrides map[string]bool
 		result.setBoolMap(&lensOverrides)
 		if result.Error == nil {
-			if o.EnabledCodeLens == nil {
-				o.EnabledCodeLens = make(map[string]bool)
+			if o.Codelens == nil {
+				o.Codelens = make(map[string]bool)
 			}
 			for lens, enabled := range lensOverrides {
-				o.EnabledCodeLens[lens] = enabled
+				o.Codelens[lens] = enabled
 			}
 		}
 
 	case "staticcheck":
-		result.setBool(&o.StaticCheck)
+		result.setBool(&o.Staticcheck)
 
 	case "local":
-		result.setString(&o.LocalPrefix)
+		result.setString(&o.Local)
 
 	case "verboseOutput":
 		result.setBool(&o.VerboseOutput)
@@ -573,6 +688,9 @@ func (o *Options) set(name string, value interface{}) OptionResult {
 	case "expandWorkspaceToModule":
 		result.setBool(&o.ExpandWorkspaceToModule)
 
+	case "experimentalWorkspaceModule":
+		result.setBool(&o.ExperimentalWorkspaceModule)
+
 	// Replaced settings.
 	case "experimentalDisabledAnalyses":
 		result.State = OptionDeprecated
